#!/bin/sh
# A Sample API for a web-login system, demonstrating haserldb 

# Set defaults, if not set previously
USERDB=${USERDB:="/var/www-db/users.tdb"}
SESSIONDB=${SESSIONDB:="/var/www-db/sessions.tdb"}
DISABLEDB=${DISABLEDB:="/var/www-db/disabled.tdb"}

# Checks the user.tdb against the stored password
# takes 1. userid 2. password
# Returns the userid if the password matches; nothing otherwise
checkpasswd () {
	[ -z "$1" ] && exit
	[ -z "$2" ] && exit

	cat <<-EOF | haserldb
	$USERDB readonlydb	# open as readonly
	datafromkey
	"${1}.passwd" get	# get userid.passwd hash
	
	count			# count how many matches we got 
	1 !=			# anything other than "one" is a problem, 
	ENDERROR ifgoto		# so exit now

	
	${2} "^" ${1} sub	# prepend the userid as salt	
	sha256			# hash the salt and password
	!=			# If they are not equal, then the user gave
	ENDERROR ifgoto		# the wrong password, exit

	${1}			# push the username onto the stack
	quit			# and quit
	
	ENDERROR label
	empty			# return null string
	EOF
	}


# Checks the user.tdb against the userid
# Takes 1. userid
# Returns the userid if the userid exists, nothing on error
checkuserid () {
	[ -z "$1" ] && exit
	
	cat <<-EOF | haserldb
	$USERDB readonlydb	# open as readonly
	keyfromkey		# we want the key (and we have the key)
	"${1}.passwd" get	# Get userid.passswd hash
	
	count			# how many responses
	1 != 			# anything other than one is an error
	ENDERROR ifgoto
	
	".passwd$" "" sub	# strip ".passwd"
	quit			# and exit
		
	ENDERROR label
	empty			# return nothing
	EOF
	}


# Adds the user to the session table; if the user is already there,
# makes a new session for them
# Takes: 1. userid 
# Returns: The Session hash, or nothing on error
addsession () {
	[ -z "$1" ] && exit

	cat <<-EOF | haserldb -d $SESSIONDB
	$1\..*$	delete		# delete all the existing user info
	
	64 randhash		# generate a rand hash
	${1}.sessid store	# and store it as sessid
	
	now 600 +		# 10 minutes from now
	${1}.expires store	# and store the expire time
		
	${1}.sessid get		# get the sessionid from the table
	EOF
	}

# Check if a session is valid
# Takes:  1. sessionid
# Returns - userid if session is valid; nothing if not
checksession () {
	if [ -z "$1" ]; then exit; fi
	
	cat <<-EOF | haserldb -d $SESSIONDB
	keyfromdata		# get the key we have the session
	${1} get
	count			
	1 !=			# if we did not get one item back
	ERROR ifgoto		# goto error
	
	"\.sessid$" "" sub	# strip the .sessid from the userid
	dup			# make a copy for later
	
	$ .expires sub		# add .expires to userid
	datafromkey		# switch to getting the data
	get			# and get the expires time
	ERROR ifempty		# if no expires time, we have problems
	now "<"			# is current time > than expire time?
	ERROR ifgoto 		# If so, the session is expired
	
	quit                    # Otherwise, the userid is on the stack, 
				# and life is good
	
	ERROR label		# we get here if there was a problem
	empty
	quit
	EOF
	}

# Given a valid session, it extends the validity time to +10minutes
# Takes: 1. sessionid
# Returns: The sessionid, if valid
extendsession () {
	[ -z "$1" ] && exit
	
	cat <<-EOF | haserldb -d $SESSIONDB
	keyfromdata
	${1} get		# get the session
	END ifclear
	"\.sessid$" "" sub	# get the userid
	"$" .expires sub	
	now 600 +		# 10 minutes from now
	swap
	store			# and store it
	${1}
	
	END label
	EOF	
	}


# Deletes a userid from the session table
# Takes: 1. userid
# Returns: Session hash, if one was deleted
deletesession () {
	[ -z "$1" ] && exit
	
	cat <<-EOF | haserldb -d $SESSIONDB
	^${1}\.sessid get		# get the existing key
	^${1}\..*$ delete		# and delete everything about the user
	EOF
	}

# Expires old sessions
expiresessions () {
	cat <<-EOF | haserldb -d $SESSIONDB
	".*\.sessid" get	# get all the sessions
	
	TOP label		# Start of loop the stack has all
				# the sessionids
	END ifempty		# end if we have no sessions
	
	keyfromdata		# we have the session, we want the key
	get			# get the session
	"\.sessid" "" sub	# strip the sessid to get the userid
	dup			# we will need this later, make 2 copies
	
	\$ \.expires sub	# add expires to userid
	datafromkey		# get the data on next fetch

	get			# get the expires time
	now			# get current time
	">"			# if expires is in the future
	BOTTOM ifgoto		# skip the deletion

	"$" ".*$" sub		# add wildcard to userid
	delete			# and delete it all
	TOP goto		# start the loop again
	
	BOTTOM label		# No deletion needed, delete the userid
	pop			
	TOP goto		# start the loop again	
	
	END label		# all done
	EOF
	}

# Checks the disabled table to see if the key is disabled
# takes 1. key (typically a userid or sourceip)
# Returns nothing if all is ok; expire timestamp if account is locked
checkdisabled() {
	[ -z "$1" ] && exit
	
	cat <<-EOF | haserldb -d $DISABLEDB
	"${1}.expires" get	# The expiration time
	END ifempty		# no expire info, exit

	dup			# make a copy for later
	now			# get now time	
	<			# is expire < now 
	EXPIRE ifgoto
	quit			# expire time is left on stack
	
	EXPIRE label		# we go here if expire time is in the past
	"${1}.expires" delete	# remove the expire record
	empty			# and clear the stack
	
	END label
	EOF

}

# Adds a timestamp for an invalid login attempt;  
# if the list of offenses is great enough, locks the account
addinvalidlogin () {
	[ -z "$1" ] && exit
		
	# get list of previous offenses and add this one
	offenses="$( haserldb -d $DISABLEDB "${1}.offense" get now )"
	# get (up to) the last 10 offenses within the last 30 minutes
	b=$(for a in $offenses; do 
		# delete offsenses > 30 minutes ago
		c=$( haserldb "now 1800 - $a <" )
		if [ "$c" = "true" ]; then  echo $a; fi
		done | sort -n | tail -n10 )

	# 5 offenses we lock for 5 minutes
	# 10 offenses we lock for 30 minutes
	offenses=$( echo $b | wc -w )
	lock=$( haserldb $offenses 4 \> 5 0 if $offenses 9 \> 10 0 if max)
	# We think we need to lock
	if [ "$lock" != "0" ]; then
		haserldb -d $DISABLEDB now $lock 60 \* + ${1}.expires store
		fi	 	
			
	# and save the offsenses list
	haserldb -d $DISABLEDB \"$b\" ${1}.offense store
	}

#-------------------------------------------------------------

